/*

$Id: misc.c,v 1.4 2008/12/19 16:51:45 alexsisson Exp $

(C) Copyright 2005-2008 Alex Sisson (alexsisson@gmail.com)

This file is part of bosh.

bosh is free software; you can redistribute it and/or modify
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

bosh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with bosh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "misc.h"

char *strdup2(const char *s1, const char *s2) {
  char *r;
  r = malloc(strlen(s1)+strlen(s2)+1);
  if(r) {
    sprintf(r,"%s%s",s1,s2);
  }
  return r;
}

/* makes a \0 terminated string of length len filled with c */
char *strmak(size_t len, int c) {
  static char *buf = NULL;
  free(buf);
  buf = malloc(len+1);
  if(buf) {
    memset(buf,c,len);
    buf[len] = 0;
  }
  return buf;
}

/* creates optstring arg (short opts) from option structure */
char *getopt_mkoptstring(struct option *longopts)
{
  int n = 0;
  char *optstring,*p;
  struct option *o = longopts;

  while(o->name || o->has_arg || o->flag || o->val) {
    n += 1 + o->has_arg;
    if(o->flag)
      return NULL; /* fail because not returning o->val */
    o++;
  }

  p = optstring = malloc(n+1);
  o = longopts;

  while(o->name || o->has_arg || o->flag || o->val) {
    *p++ = o->val;
    n = o->has_arg;
    while(n--)
      *p++ = ':';
    o++;
  }
  *p = 0;

  return optstring;
}

