/*

Config file related routines.

$Id: rc.c,v 1.36 2008/12/19 16:52:32 alexsisson Exp $

(C) Copyright 2002-2008 Alex Sisson (alexsisson@gmail.com)

This file is part of bosh.

bosh is free software; you can redistribute it and/or modify
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

bosh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with bosh; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "rc.h"
#include "misc.h"
#include "action.h"


/* Reads a configuration, processes the actions, and stores other options into the arg array */

int rc_read(char *name, BOSH *bosh, int *pargc, char ***pargv) {

  FILE *f;
  char s[256],*p,*a,*v;
  int i, line=0, err=0;
  int argc = 1;
  char **argv = malloc(sizeof(char*)*(*pargc+32));
  argv[0] = *pargv[0];

  /* open config */
  f = fopen(name,"r");
  if(!f) {
    fprintf(stderr,"%s: %s: file not found\n",PACKAGE,name);
    exit(1);
  }

  while(!feof(f)) {
    if(!fgets(s,sizeof(s),f)) {
      if(ferror(f)) {
        fprintf(stderr,"%s: %s:%02d: read error\n",PACKAGE,conf,line);
        err = 1;
      }
      break;  
    }

    line++;

    /* it's a comment */
    if(*s=='#')
      continue;

    /* lose \n */
    p = strchr(s,'\n');
    if(p)
      *p = 0;

    /* it's an empty line */
    if(!*s)
      continue;

    /* try and find equals-sign */
    p = strchr(s,'=');
    if(!p) {
      fprintf(stderr,"%s: %s:%02d: configuration error: = expected.\n",PACKAGE,conf,line);
      err = 1;
      break;
    }
    *p = 0;
    p++;

    a = strdup(s); /* attribute */
    v = strdup(p); /* value */

    /* handle value on next line */
    if(!*v) {
      if(!fgets(s,sizeof(s),f)) {
        if(ferror(f)) {
          fprintf(stderr,"%s: %s:%02d: read error\n",PACKAGE,conf,line);
          err = 1;
        }
        break;
      }
      free(v);
      if(*s=='#')
        continue;
      p = strchr(s,'\n');
      if(p)
        *p = 0;
      v = strdup(s);
    }

    /* handle multi-line value */
    while(v[strlen(v)-1]=='\\') {
      v[strlen(v)-1]='\n';
      if(!fgets(s,sizeof(s),f)) {
        if(ferror(f)) {
          fprintf(stderr,"%s: %s:%02d: read error\n",PACKAGE,conf,line);
          err = 1;
        }
        break;
      }
      line++;
      if(*s=='#')
        continue;
      p = strchr(s,'\n');
      if(p)
        *p = 0;
      v = realloc(v,strlen(v)+strlen(s)+1);
      strcat(v,s);
    }

    /* process */
    if(strlen(a)==1) {
      /* action */
      sprintf(s,"%s=%s",a,v);
      switch(action_set(s)) {
        case -1:
//          fprintf(
          break;
        case -2:
           break;
        default:
          break;
      }
    } else {
      /* else copy it into our argv so it can be parsed as an argument */
      argv[argc] = malloc(strlen(a)+strlen(v)+4); /* --(a)=(v)\0 */
      sprintf(argv[argc],"--%s=%s",a,v);
      argc++;
    }

    free(a);
    free(v);
  }

  fclose(f);

  if(err)
    exit(1);

  /* copy original command line arguments into our new argv */
  for(i=1;i<*pargc;i++) {
    argv[argc] = (*pargv)[i];
    argc++;
  }

  /* update reference args */
  *pargc = argc;
  *pargv = argv;
  
  return 0;
}

int rc_write(char *name, BOSH *bosh) {

/*
  FILE *f;
  char s[256];
  int i;

  strcpy(s,getenv("HOME"));
  strcat(s,"/.bosh/");
  strcat(s,name);

  f = fopen(s,"w");
  if(!f)
    return -1;

  fprintf(f,"command=%s\n",bosh->command);

  if(bosh->refresh)
    fprintf(f,"refresh=%C\n",bosh->refresh+48);
    
  fputc('\n',f);

  for(i=0;i<10;i++)
    if(bosh->action[i].command)
      fprintf(f,"%C=%s\n",i+48,bosh->action[i].command);

  for(i=10;i<36;i++)
    if(bosh->action[i].command)
      fprintf(f,"%C=%s\n",i+55,bosh->action[i].command);

  if(bosh->action[37])
    fprintf(f,"enter=%s\n",bosh->action[37]);

  fputc('\n',f);

  fclose(f);
*/
  return 0;
}
